import os
import glob
import shutil
import argparse


def force_create_folder(folder_to_create):
    if os.path.isdir(folder_to_create):
        shutil.rmtree(folder_to_create, ignore_errors=True)
        if os.path.isdir(folder_to_create):
            shutil.rmtree(folder_to_create, ignore_errors=True)
            if os.path.isdir(folder_to_create):
                shutil.rmtree(folder_to_create, ignore_errors=True)
                if os.path.isdir(folder_to_create):
                    shutil.rmtree(folder_to_create, ignore_errors=True)
    os.mkdir(folder_to_create)


def rename_mag(args):

    output_prefix   = args['p']
    mag_folder      = args['m']
    mag_ext         = args['mx']
    ref_folder      = args['r']
    ref_ext         = args['rx']
    ani_cutoff      = args['c']
    num_thread      = args['t']


    ################################################### define file name ###################################################

    link_wd = '%s_link_mag_ref_wd' % output_prefix

    mag_folder_no_path = mag_folder
    if '/' in mag_folder:
        mag_folder_no_path = mag_folder.split('/')[-1]

    ref_folder_no_path = ref_folder
    if '/' in ref_folder:
        ref_folder_no_path = ref_folder.split('/')[-1]

    mag_list_file       = '%s/%s_%s_file_list.txt'      % (link_wd, output_prefix, mag_folder_no_path)
    ref_list_file       = '%s/%s_%s_file_list.txt'      % (link_wd, output_prefix, ref_folder_no_path)
    fastANI_output      = '%s/%s_fastANI.txt'           % (link_wd, output_prefix)
    ref_to_mag_table    = '%s/%s_ref_to_mag_table.txt'  % (link_wd, output_prefix)
    renamed_to_file     = '%s/%s_renamed_to.txt'        % (link_wd, output_prefix)
    renamed_mag_folder  = '%s/%s_renamed'               % (link_wd, mag_folder_no_path )

    force_create_folder(link_wd)


    ############################################### prepare files for fastANI ##############################################

    # get file list
    mag_file_re = '%s/*.%s' % (mag_folder, mag_ext)
    ref_file_re = '%s/*.%s' % (ref_folder, ref_ext)
    mag_file_list = [os.path.basename(file_name) for file_name in glob.glob(mag_file_re)]
    ref_file_list = [os.path.basename(file_name) for file_name in glob.glob(ref_file_re)]

    mag_list_file_handle = open(mag_list_file, 'w')
    for mag_file in mag_file_list:
        mag_list_file_handle.write('%s/%s\n' % (mag_folder, mag_file))
    mag_list_file_handle.close()

    ref_list_file_handle = open(ref_list_file, 'w')
    for ref_file in ref_file_list:
        ref_list_file_handle.write('%s/%s\n' % (ref_folder, ref_file))
    ref_list_file_handle.close()

    fastANI_cmd = 'fastANI --refList %s --ql %s -o %s --minFrag 1 -t %s' % (ref_list_file, mag_list_file, fastANI_output, num_thread)
    os.system(fastANI_cmd)


    ################################################# parse fastANI output #################################################

    os.mkdir(renamed_mag_folder)

    ref_to_query_dict = {}
    query_to_ref_dict = {}
    for line in open(fastANI_output):

        line_split = line.strip().split('\t')
        query_name = line_split[0]
        if '/' in query_name:
            query_name = query_name.split('/')[-1]

        ref_name   = line_split[1]
        if '/' in ref_name:
            ref_name = ref_name.split('/')[-1]

        ani_value  = float(line_split[2])

        if ani_value >= ani_cutoff:

            if ref_name not in ref_to_query_dict:
                ref_to_query_dict[ref_name] = [query_name]
            else:
                ref_to_query_dict[ref_name].append(query_name)

            if query_name not in query_to_ref_dict:
                query_to_ref_dict[query_name] = [ref_name]
            else:
                query_to_ref_dict[query_name].append(ref_name)


    ref_to_mag_table_handle = open(ref_to_mag_table, 'w')
    renamed_to_file_handle = open(renamed_to_file, 'w')
    ref_list_match_to_multiple_mag = []
    for ref in ref_to_query_dict:

        ref_no_ext                  = '.'.join(ref.split('.')[:-1])
        current_query_list          = sorted(ref_to_query_dict[ref])
        current_query_list_no_ext   = ['.'.join(i.split('.')[:-1]) for i in current_query_list]

        ref_to_mag_table_handle.write('%s\t%s\n' % (ref_no_ext, ','.join(current_query_list_no_ext)))

        # rename mags
        if len(current_query_list) == 1:
            rename_cmds = 'cp %s/%s %s/%s' % (mag_folder, current_query_list[0], renamed_mag_folder, ref)
            renamed_to_file_handle.write('%s\t%s\n' % ('.'.join(current_query_list[0].split('.')[:-1]), ref_no_ext))
            os.system(rename_cmds)
        elif len(current_query_list) > 1:
            rename_index = 1
            for each_query in current_query_list:
                rename_cmds = 'cp %s/%s %s/%s_%s.%s' % (mag_folder,  each_query, renamed_mag_folder,  ref_no_ext, rename_index, ref_ext)
                renamed_to_file_handle.write('%s\t%s_%s\n' % ('.'.join(each_query.split('.')[:-1]), ref_no_ext, rename_index))
                os.system(rename_cmds)
                rename_index += 1
            ref_list_match_to_multiple_mag.append(ref_no_ext)

    ref_to_mag_table_handle.close()
    renamed_to_file_handle.close()


    ######################################################## report ########################################################

    mags_matched_to_multi_refs_list = []
    for mag in query_to_ref_dict:
        if len(query_to_ref_dict[mag]) >= 2:
            mags_matched_to_multi_refs_list.append(mag)
    if len(mags_matched_to_multi_refs_list) == 0:
        print('Good! No MAG matched to multiple reference genomes.')
    else:
        print('Caution! Found MAG(s) matched to multiple reference genomes.')


    ref_list_not_match_to_mag = []
    for each_ref in ref_file_list:
        if each_ref not in ref_to_query_dict:
            ref_list_not_match_to_mag.append(each_ref)
    if len(ref_list_not_match_to_mag) == 0:
        print('Good! All reference(s) have matched MAGs')
    else:
        print('Caution! Found reference(s) do not have matched MAG(%s): %s' % (len(ref_list_not_match_to_mag), ','.join(ref_list_not_match_to_mag)))


    if len(ref_list_match_to_multiple_mag) == 0:
        print('Good! No reference matched to multiple MAGs')
    else:
        print('Caution! Found reference(s) matched to multiple MAG(%s): %s' % (len(ref_list_match_to_multiple_mag), ','.join(sorted(ref_list_match_to_multiple_mag))))


if __name__ == '__main__':

    rename_mag_parser = argparse.ArgumentParser()

    rename_mag_parser.add_argument('-p',             required=True,                              help='output prefix')
    rename_mag_parser.add_argument('-m',             required=True,  default=None,               help='MAG folder')
    rename_mag_parser.add_argument('-mx',            required=False, default='fasta',            help='MAG file extension, default: fasta')
    rename_mag_parser.add_argument('-r',             required=True,  default=None,               help='REF folder')
    rename_mag_parser.add_argument('-rx',            required=False, default='fasta',            help='REF file extension, default: fasta')
    rename_mag_parser.add_argument('-c',             required=False, type=float, default=99,     help='ANI cutoff, default: 99')
    rename_mag_parser.add_argument('-t',             required=False, type=int, default=1,        help='number of threads, default: 1')

    args = vars(rename_mag_parser.parse_args())

    rename_mag(args)


'''

module load python/3.7.3
source ~/mypython3env/bin/activate
module load fastani/1.1
cd /srv/scratch/z5039045/test
python3 rename_mag.py -p Test -m Refined_refined_bins -mx fasta -r selected_genomes_renamed_no_plasmid -rx fna -c 99 -t 12

'''



from Bio import SeqIO


def prefix_seq(seq_in, prefix, seq_out):

    seq_out_handle = open(seq_out, 'w')
    for seq_record in SeqIO.parse(seq_in, 'fasta'):
        seq_id_new = '%s_%s' % (prefix, seq_record.id)
        seq_out_handle.write('>%s\n' % seq_id_new)
        seq_out_handle.write('%s\n' % str(seq_record.seq))
    seq_out_handle.close()


matam_assemblies            = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI_Oral/CAMI_Oral_Matam16S_wd/CAMI_Oral_16S_reads_subset_50_Matam_wd/workdir/scaffolds.NR.min_500bp.fa'
matam_assemblies_prefixed   = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI_Oral/CAMI_Oral_Matam16S_wd/CAMI_Oral_16S_reads_subset_50_Matam_wd/workdir/scaffolds.NR.min_500bp.prefixed.fa'
seq_prefix                  = '%s_subsample_%s' % ('CAMI_Oral', 50)

prefix_seq(matam_assemblies, seq_prefix, matam_assemblies_prefixed)






def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if each_element.isalpha() is True:
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted



pwd_samfile                         = '/Users/songweizhi/Desktop/BH_ER_050417_assembled_16S_uclust_0.995.sam'
clipping_reads_not_matched_part_seq = '/Users/songweizhi/Desktop/clipping_reads_not_matched_part.fasta'
mismatch_ratio_max_value = 0.5
perfect_match_min_cigar_M_pct = 80
perfect_match_max_cigar_S_pct = 20


# export clipping mapped reads and perfectly mapped reads
all_mapped_reads_set = set()
clipping_mapped_reads_list = set()
clipping_reads_mapped_part_dict = {}
perfectly_mapped_reads_dict = {}
clipping_reads_not_matched_part_seq_handle = open(clipping_reads_not_matched_part_seq, 'w')
for each_read in open(pwd_samfile):

    if not each_read.startswith('@'):
        each_read_split = each_read.strip().split('\t')
        read_id = each_read_split[0]
        read_id_base = '.'.join(read_id.split('.')[:-1])
        read_strand = read_id.split('.')[-1]
        ref_id = each_read_split[2]
        ref_id_with_prefix = 'MarkerGene__%s' % each_read_split[2]
        if ref_id_with_prefix == 'MarkerGene__BH_ER_050417_subsample_50_354':
            print(each_read.strip())
        ref_pos = int(each_read_split[3])
        cigar = each_read_split[5]
        read_seq = each_read_split[9]
        cigar_splitted = cigar_splitter(cigar)
        read_id_with_ref_pos = '%s__x__%s__x__%s' % (read_id, ref_id, ref_pos)
        all_mapped_reads_set.add(read_id)
        treat_as_full_match = False

        # for perfectly mapped reads, e.g. 150M
        if ('M' in cigar) and (len(cigar_splitted) == 1):
            treat_as_full_match = True

        # for perfectly mapped reads, e.g. 120M-1D-80M
        if len(cigar_splitted) == 3:
            if (cigar_splitted[0][-1] == 'M') and (cigar_splitted[2][-1] == 'M'):
                mismatch_ratio = int(cigar_splitted[1][:-1]) * 100 / len(read_seq)
                M_pct_l = int(cigar_splitted[0][:-1]) * 100 / len(read_seq)
                M_pct_r = int(cigar_splitted[2][:-1]) * 100 / len(read_seq)
                if (M_pct_l >= 20) and (M_pct_r >= 20) and (mismatch_ratio <= 0.5):
                    treat_as_full_match = True

        if ('S' in cigar) and (len(cigar_splitted) == 2):
            cigar_M_len = 0
            cigar_S_len = 0
            if cigar_splitted[0][-1] == 'M':
                cigar_M_len = int(cigar_splitted[0][:-1])
                cigar_S_len = int(cigar_splitted[1][:-1])
            if cigar_splitted[1][-1] == 'M':
                cigar_M_len = int(cigar_splitted[1][:-1])
                cigar_S_len = int(cigar_splitted[0][:-1])

            cigar_M_pct = cigar_M_len * 100 / (cigar_M_len + cigar_S_len)
            cigar_S_pct = cigar_S_len * 100 / (cigar_M_len + cigar_S_len)

            # for clipping reads with unmapped part >= min_cigar_S
            if (cigar_M_pct >= 30) and (cigar_S_pct >= 30):
                read_seq_left = read_seq[: int(cigar_splitted[0][:-1])]
                read_seq_right = read_seq[-int(cigar_splitted[1][:-1]):]

                if cigar_splitted[0][-1] == 'M':

                    # write out the sequence of unmapped part
                    clipping_reads_not_matched_part_seq_handle.write('>%s_r\n' % read_id_with_ref_pos)
                    clipping_reads_not_matched_part_seq_handle.write(read_seq_right + '\n')

                    # store the match info of mapped part
                    if ('%s_l' % read_id_with_ref_pos) not in clipping_reads_mapped_part_dict:
                        clipping_reads_mapped_part_dict[('%s_l' % read_id_with_ref_pos)] = [ref_id_with_prefix]
                    else:
                        clipping_reads_mapped_part_dict[('%s_l' % read_id_with_ref_pos)].append(ref_id_with_prefix)

                if cigar_splitted[1][-1] == 'M':

                    # write out the sequence of unmapped part
                    clipping_reads_not_matched_part_seq_handle.write('>%s_l\n' % read_id_with_ref_pos)
                    clipping_reads_not_matched_part_seq_handle.write(read_seq_left + '\n')

                    # store the match info of mapped part
                    if ('%s_r' % read_id_with_ref_pos) not in clipping_reads_mapped_part_dict:
                        clipping_reads_mapped_part_dict[('%s_r' % read_id_with_ref_pos)] = [ref_id_with_prefix]
                    else:
                        clipping_reads_mapped_part_dict[('%s_r' % read_id_with_ref_pos)].append(ref_id_with_prefix)

                clipping_mapped_reads_list.add(read_id)

            # for clipping reads with unmapped part < min_cigar_S
            # treat these reads as perfectly mapped reads
            elif (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (cigar_S_pct < perfect_match_max_cigar_S_pct):
                if read_id_base not in perfectly_mapped_reads_dict:
                    perfectly_mapped_reads_dict[read_id_base] = {read_strand: [ref_id_with_prefix]}
                else:
                    if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                        perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                    else:
                        perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)

        elif ('S' in cigar) and (len(cigar_splitted) > 2):

            if ((cigar_splitted[0][-1] == 'M') and (cigar_splitted[-1][-1] == 'S')) or \
                    ((cigar_splitted[0][-1] == 'S') and (cigar_splitted[-1][-1] == 'M')):

                if len(cigar_splitted) == 4:

                    # e.g. 30M-1D-50M-40S
                    if (cigar_splitted[0][-1] == 'M') and (cigar_splitted[2][-1] == 'M') and (
                            cigar_splitted[3][-1] == 'S'):
                        mismatch_ratio = int(cigar_splitted[1][:-1]) * 100 / len(read_seq)
                        if (mismatch_ratio <= mismatch_ratio_max_value) and (int(cigar_splitted[2][:-1]) >= 30):
                            cigar_M_pct = (int(cigar_splitted[0][:-1]) + int(cigar_splitted[2][:-1])) * 100 / len(
                                read_seq)
                            cigar_S_pct = (int(cigar_splitted[3][:-1])) * 100 / len(read_seq)
                            if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (
                                    cigar_S_pct < perfect_match_max_cigar_S_pct):
                                treat_as_full_match = True

                    # e.g. 40S-30M-1D-50M
                    if (cigar_splitted[0][-1] == 'S') and (cigar_splitted[1][-1] == 'M') and (
                            cigar_splitted[3][-1] == 'M'):
                        mismatch_ratio = int(cigar_splitted[2][:-1]) * 100 / len(read_seq)
                        if (mismatch_ratio <= mismatch_ratio_max_value) and (int(cigar_splitted[1][:-1]) >= 30):
                            cigar_M_pct = (int(cigar_splitted[1][:-1]) + int(cigar_splitted[3][:-1])) * 100 / len(
                                read_seq)
                            cigar_S_pct = (int(cigar_splitted[0][:-1])) * 100 / len(read_seq)
                            if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (
                                    cigar_S_pct < perfect_match_max_cigar_S_pct):
                                treat_as_full_match = True

                # elif len(cigar_splitted) == 6:
                #
                #     if (cigar_splitted[0][-1] == 'M') and (cigar_splitted[2][-1] == 'M') and (cigar_splitted[4][-1] == 'M') and (cigar_splitted[5][-1] == 'S'):
                #         mismatch_ratio = (int(cigar_splitted[1][:-1]) + int(cigar_splitted[3][:-1])) * 100 / len(read_seq)
                #         if mismatch_ratio <= mismatch_ratio_max_value:
                #             cigar_M_pct = (int(cigar_splitted[0][:-1]) + int(cigar_splitted[2][:-1]) + int(cigar_splitted[4][:-1])) * 100 / len(read_seq)
                #             cigar_S_pct = (int(cigar_splitted[5][:-1])) * 100 / len(read_seq)
                #             if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (
                #                     cigar_S_pct < perfect_match_max_cigar_S_pct):
                #                 treat_as_full_match = True
                #
                #     if (cigar_splitted[0][-1] == 'S') and (cigar_splitted[1][-1] == 'M') and (cigar_splitted[3][-1] == 'M') and (cigar_splitted[5][-1] == 'M'):
                #         mismatch_ratio = (int(cigar_splitted[2][:-1]) + int(cigar_splitted[4][:-1])) * 100 / len(read_seq)
                #         if mismatch_ratio <= mismatch_ratio_max_value:
                #             cigar_M_pct = (int(cigar_splitted[1][:-1]) + int(cigar_splitted[3][:-1]) + int(cigar_splitted[5][:-1])) * 100 / len(read_seq)
                #             cigar_S_pct = (int(cigar_splitted[0][:-1])) * 100 / len(read_seq)
                #             if (cigar_M_pct >= perfect_match_min_cigar_M_pct) and (cigar_S_pct < perfect_match_max_cigar_S_pct):
                #                 treat_as_full_match = True

        if treat_as_full_match is True:

            if ref_id_with_prefix == 'MarkerGene__BH_ER_050417_subsample_50_354':
                print(cigar_splitted)
                print()


            if read_id_base not in perfectly_mapped_reads_dict:
                perfectly_mapped_reads_dict[read_id_base] = {read_strand: [ref_id_with_prefix]}
            else:
                if read_strand not in perfectly_mapped_reads_dict[read_id_base]:
                    perfectly_mapped_reads_dict[read_id_base][read_strand] = [ref_id_with_prefix]
                else:
                    perfectly_mapped_reads_dict[read_id_base][read_strand].append(ref_id_with_prefix)

clipping_reads_not_matched_part_seq_handle.close()

perfectly_mapped_read_singleton_dict = {}
for perfectly_mapped_read in perfectly_mapped_reads_dict:
    current_value = perfectly_mapped_reads_dict[perfectly_mapped_read]
    if len(current_value) == 1:
        perfectly_mapped_read_singleton_dict[perfectly_mapped_read] = current_value


print(perfectly_mapped_read_singleton_dict)






demo_dict = {'a':'1', 'b':2, 'c':3}

demo_set = {'a', 'c'}

set_a = {'1','2',3}
set_b = {3,'5','6'}

set_c = set.union(set_a, set_b)


print(set_a)
print(set_b)
print(set_c)
print(set_a)
print(set_b)


quality_folder = '/Users/songweizhi/Desktop/bin_quality_folder'

for each_uc in open('/Users/songweizhi/Desktop/32_unclassified.tsv'):
    #print(each_uc)
    mag_id = each_uc.split('\t')[0]
    #print(mag_id)
    report_str = each_uc.split('\t')[1].strip()
    #print(report_str)
    sample_id = mag_id.split('_Refined_')[0]
    mag_id_no_sample_id = 'Refined_' + mag_id.split('_Refined_')[1].strip()
    #print(mag_id_no_sample_id)

    pwd_quality_file = '%s/%s_Refined_qualified.txt' % (quality_folder, sample_id)

    mag_quality_str = ''
    for each_qual in open(pwd_quality_file):
        each_qual_split = each_qual.strip().split('\t')
        bin_id = each_qual.split('\t')[0]
        if bin_id == mag_id_no_sample_id:
            mag_quality_str = '%s\t%s' % (each_qual_split[3], each_qual_split[4])
    #print(mag_quality_str)



    print('%s\t%s' % (each_uc.strip(), mag_quality_str))


import os

wd               = '/Users/songweizhi/Desktop/MarkerMAG_wd/Figures/16S_tree'
gene_list_file   = '%s/gene_list.txt'                                           % wd
rank_list        = ['d', 'p', 'c', 'o', 'f', 'g', 's']
ranks_with_color = ['d', 'p', 'c', 'o', 'f', 'g']

gene_list = []
for gene in open(gene_list_file):
    gene_list.append(gene.strip())


for taxon_rank in rank_list:
    print(taxon_rank)
    genome_to_taxon     = '%s/16S_grouping_rank_%s.txt'                 % (wd, taxon_rank)
    gene_to_taxon       = '%s/16S_grouping_rank_%s_gene_level.txt'      % (wd, taxon_rank)
    gene_to_taxon_itol  = '%s/16S_grouping_rank_%s_gene_level_iTOL.txt' % (wd, taxon_rank)
    group_to_color      = '%s/16S_group_color_%s.txt'                   % (wd, taxon_rank)

    genome_to_taxon_dict = {}
    for genome_taxon in open(genome_to_taxon):
        genome_taxon_split = genome_taxon.strip().split('\t')
        genome_to_taxon_dict[genome_taxon_split[0]] = '_'.join(genome_taxon_split[1].split('_'))

    gene_to_taxon_handle = open(gene_to_taxon, 'w')
    for gene in sorted(gene_list):
        gene_to_taxon_handle.write('%s\t%s\n' % (gene, genome_to_taxon_dict[gene[:2]]))
    gene_to_taxon_handle.close()

    BioSAK_cmd = 'BioSAK iTOL -ColorStrip -lg %s -lt %s -out %s' % (gene_to_taxon, taxon_rank, gene_to_taxon_itol)
    if taxon_rank in ranks_with_color:
        BioSAK_cmd = 'BioSAK iTOL -ColorStrip -lg %s -gc %s -lt %s -out %s' % (gene_to_taxon, group_to_color, taxon_rank, gene_to_taxon_itol)

    if taxon_rank == 's':
        #BioSAK_cmd = 'BioSAK iTOL -ColorStrip -lg %s -gc %s -lt %s -out %s' % (gene_to_taxon, group_to_color, taxon_rank, gene_to_taxon_itol)
        BioSAK_cmd = 'BioSAK iTOL -ColorRange -lg %s -gc %s -lt %s -out %s' % (gene_to_taxon, group_to_color, taxon_rank, gene_to_taxon_itol)

    os.system(BioSAK_cmd)

    os.remove(gene_to_taxon)



from Bio import SeqIO

# ref_gnm_id_txt = 'ref_gnm_id.txt'
#
# for each_ref in open(ref_gnm_id_txt):
#     gnm_id = each_ref.strip()
#     pwd_gnm_old = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/ref_genomes_hc_04_20/%s' % gnm_id
#     pwd_gnm_new = '/srv/scratch/z5039045/MarkerMAG_wd/CAMI1/ref_genomes_hc_04_20_uniq/%s' % gnm_id
#
#     wrote_ctgs = set()
#     pwd_gnm_new_handle = open(pwd_gnm_new, 'w')
#     for each_ctg in SeqIO.parse(pwd_gnm_old, 'fasta'):
#         if each_ctg.id not in wrote_ctgs:
#             SeqIO.write(each_ctg, pwd_gnm_new_handle, 'fasta')
#             wrote_ctgs.add(each_ctg.id)
#     pwd_gnm_new_handle.close()

fa_file = '/Users/songweizhi/Desktop/NODE_345_reads.fasta'

fa_file_r1 = '/Users/songweizhi/Desktop/NODE_345_reads_R1.fasta'
fa_file_r2 = '/Users/songweizhi/Desktop/NODE_345_reads_R2.fasta'
fa_file_up = '/Users/songweizhi/Desktop/NODE_345_reads_UP.fasta'


seq_id_set = set()
for each_read in SeqIO.parse(fa_file, 'fasta'):
    seq_id_set.add(each_read.id)

print(seq_id_set)

seq_id_set_no_strand = {'.'.join(i.split('.')[:-1]) for i in seq_id_set}

print(seq_id_set_no_strand)

paired_reads = set()
for each_base_name in seq_id_set_no_strand:
    r1 = '%s.1' % each_base_name
    r2 = '%s.2' % each_base_name
    if (r1 in seq_id_set) and (r2 in seq_id_set):
        paired_reads.add(r1)
        paired_reads.add(r2)

fa_file_r1_handle = open(fa_file_r1, 'w')
fa_file_r2_handle = open(fa_file_r2, 'w')
fa_file_up_handle = open(fa_file_up, 'w')
for each_read in SeqIO.parse(fa_file, 'fasta'):
    if each_read.id in paired_reads:
        if each_read.id[-1] == '1':
            fa_file_r1_handle.write('>%s\n' % each_read.id)
            fa_file_r1_handle.write('%s\n' % each_read.seq)
        if each_read.id[-1] == '2':
            fa_file_r2_handle.write('>%s\n' % each_read.id)
            fa_file_r2_handle.write('%s\n' % each_read.seq)
    else:
        fa_file_up_handle.write('>%s\n' % each_read.id)
        fa_file_up_handle.write('%s\n' % each_read.seq)
fa_file_r1_handle.close()
fa_file_r2_handle.close()
fa_file_up_handle.close()


import os
import glob
import shutil
import argparse
import numpy as np
import pandas as pd
import seaborn as sns
from Bio import SeqIO
from Bio.Seq import Seq
import multiprocessing as mp
from datetime import datetime
import plotly.graph_objects as go
from Bio.SeqRecord import SeqRecord
from distutils.spawn import find_executable


class MappingRecord:

    #  sequences store in r1_seq and r2_seq should NOT been reverse complemented

    def __init__(self):

        self.r1_seq = ''
        self.r2_seq = ''

        self.r1_refs = dict()
        self.r2_refs = dict()

        self.r1_cigar_to_flag = dict()
        self.r2_cigar_to_flag = dict()

        self.r1_longest_clp_cigar = ''
        self.r1_longest_clp_falg  = ''
        self.r2_longest_clp_cigar = ''
        self.r2_longest_clp_falg  = ''

        self.qualified_reads           = False
        self.consider_round_2          = False
        self.consider_r1_unmapped_mate = False
        self.consider_r1_clipping_part = False
        self.consider_r2_unmapped_mate = False
        self.consider_r2_clipping_part = False

        self.r1_filtered_refs = set()
        self.r2_filtered_refs = set()

        self.r1_clipping_seq = ''
        self.r2_clipping_seq = ''

        self.unmapped_r1_refs = set()
        self.unmapped_r2_refs = set()

        self.clipping_r1_refs = set()
        self.clipping_r2_refs = set()


def cigar_splitter(cigar):

    # get the position of letters
    letter_pos_list = []
    n = 0
    for each_element in cigar:
        if (each_element.isalpha() is True) or (each_element == '='):
            letter_pos_list.append(n)
        n += 1

    # split cigar
    index = 0
    cigar_splitted = []
    while index <= len(letter_pos_list) - 1:
        if index == 0:
            cigar_splitted.append(cigar[:(letter_pos_list[index] + 1)])
        else:
            cigar_splitted.append(cigar[(letter_pos_list[index - 1] + 1):(letter_pos_list[index] + 1)])
        index += 1

    return cigar_splitted


def get_cigar_stats(cigar_splitted):

    # aligned_len: M I X =
    # clipping_len: S
    # mismatch_len: X I D
    # mismatch_pct = mismatch_len / aligned_len
    # aligned_pct  = aligned_len  / (aligned_len + clipping_len)
    # clipping_pct = clipping_len / (aligned_len + clipping_len)

    aligned_len = 0
    clipping_len = 0
    mismatch_len = 0
    for each_part in cigar_splitted:
        each_part_len = int(each_part[:-1])
        each_part_cate = each_part[-1]

        # get aligned_len
        if each_part_cate in {'M', 'm', 'I', 'i', 'X', 'x', '='}:
            aligned_len += each_part_len

        # get clipping_len
        if each_part_cate in ['S', 's']:
            clipping_len += each_part_len

        # get mismatch_len
        if each_part_cate in {'I', 'i', 'X', 'x', 'D', 'd'}:
            mismatch_len += each_part_len

    aligned_pct  = float("{0:.2f}".format(aligned_len * 100 / (aligned_len + clipping_len)))
    clipping_pct = float("{0:.2f}".format(clipping_len * 100 / (aligned_len + clipping_len)))
    mismatch_pct = float("{0:.2f}".format(mismatch_len * 100 / (aligned_len)))

    return aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct


def sam_flag_to_rc(flag_value):

    read_rced = 'na'
    if flag_value != '':
        binary_flag = "{0:b}".format(int(flag_value))
        binary_flag_len = len(str(binary_flag))
        binary_flag_polished = '0' * (12 - binary_flag_len) + str(binary_flag)

        if binary_flag_polished[7] == '0':
            read_rced = False
        if binary_flag_polished[7] == '1':
            read_rced = True

    return read_rced


def get_rc(seq_in):
    seq_in_rc = str(SeqRecord(Seq(seq_in)).reverse_complement().seq)
    return seq_in_rc


def get_max_clp_and_index(r1_cigar_list, r2_cigar_list):

    r1_cigar_list_split = [cigar_splitter(i) for i in r1_cigar_list]
    r2_cigar_list_split = [cigar_splitter(i) for i in r2_cigar_list]

    r1_cigar_list_split_only_clp = []
    for each_r1_cigar_split in r1_cigar_list_split:
        clp_len_l = 0
        if each_r1_cigar_split[0][-1] in ['S', 's']:
            clp_len_l = int(each_r1_cigar_split[0][:-1])
        clp_len_r = 0
        if each_r1_cigar_split[-1][-1] in ['S', 's']:
            clp_len_r = int(each_r1_cigar_split[-1][:-1])
        r1_cigar_list_split_only_clp.append([clp_len_l, clp_len_r])

    r2_cigar_list_split_only_clp = []
    for each_r2_cigar_split in r2_cigar_list_split:
        clp_len_l = 0
        if each_r2_cigar_split[0][-1] in ['S', 's']:
            clp_len_l = int(each_r2_cigar_split[0][:-1])
        clp_len_r = 0
        if each_r2_cigar_split[-1][-1] in ['S', 's']:
            clp_len_r = int(each_r2_cigar_split[-1][:-1])
        r2_cigar_list_split_only_clp.append([clp_len_l, clp_len_r])

    cigar_list_split_only_clp_r1_r2 = [r1_cigar_list_split_only_clp, r2_cigar_list_split_only_clp]

    max_value = 0
    max_value_index = ''
    for num_list_1 in cigar_list_split_only_clp_r1_r2[0]:
        if num_list_1[0] > max_value:
            max_value = num_list_1[0]
            max_value_index = 'r1_l'
        if num_list_1[1] > max_value:
            max_value = num_list_1[1]
            max_value_index = 'r1_r'
    for num_list_2 in cigar_list_split_only_clp_r1_r2[1]:
        if num_list_2[0] > max_value:
            max_value = num_list_2[0]
            max_value_index = 'r2_l'
        if num_list_2[1] > max_value:
            max_value = num_list_2[1]
            max_value_index = 'r2_r'

    # get the best cigar
    best_cigar = ''
    if max_value_index == 'r1_l':
        if best_cigar == '':
            for each_cigar in r1_cigar_list:
                if (each_cigar.startswith('%sS' % max_value)) or (each_cigar.startswith('%ss' % max_value)):
                    best_cigar = each_cigar
    elif max_value_index == 'r1_r':
        if best_cigar == '':
            for each_cigar in r1_cigar_list:
                if (each_cigar.endswith('%sS' % max_value)) or (each_cigar.endswith('%ss' % max_value)):
                    best_cigar = each_cigar
    elif max_value_index == 'r2_l':
        if best_cigar == '':
            for each_cigar in r2_cigar_list:
                if (each_cigar.startswith('%sS' % max_value)) or (each_cigar.startswith('%ss' % max_value)):
                    best_cigar = each_cigar
    elif max_value_index == 'r2_r':
        if best_cigar == '':
            for each_cigar in r2_cigar_list:
                if (each_cigar.endswith('%sS' % max_value)) or (each_cigar.endswith('%ss' % max_value)):
                    best_cigar = each_cigar

    return best_cigar, max_value, max_value_index

wd = '/Users/songweizhi/Desktop/444'
combined_1st_round_unlinked_mags_sam_best_match = '%s/round_1_unlinked_gnm.sam' % wd
max_mis_pct_rd2 = 2
min_clp_len = 35
clp_read_for_assembly = True
min_M_len = 30
end_seq_len = 500
free_living_ctg_ref_file = '%s/free_living_ctg_refs.txt' % wd
free_living_ctg_R1 = '%s/free_living_ctg_R1.fa' % wd
free_living_ctg_R2 = '%s/free_living_ctg_R2.fa' % wd
free_living_ctg_UP = '%s/free_living_ctg_UP.fa' % wd


round_2_MappingRecord_dict = {}
for each_line in open(combined_1st_round_unlinked_mags_sam_best_match):
    each_line_split = each_line.strip().split('\t')
    if not each_line.startswith('@'):
        store_read_seq = False
        read_id = each_line_split[0]
        read_id_base = '.'.join(read_id.split('.')[:-1])
        read_strand = read_id.split('.')[-1]
        read_flag = int(each_line_split[1])
        cigar = each_line_split[5]
        read_seq = each_line_split[9]
        if cigar != '*':
            ref_id = each_line_split[2]
            ref_pos = each_line_split[3]
            ref_id_with_pos = '%s_pos_%s' % (ref_id, ref_pos)
            cigar_splitted = cigar_splitter(cigar)
            aligned_len, aligned_pct, clipping_len, clipping_pct, mismatch_pct = get_cigar_stats(cigar_splitted)
            if mismatch_pct <= max_mis_pct_rd2:

                if read_id_base not in round_2_MappingRecord_dict:
                    round_2_MappingRecord_dict[read_id_base] = MappingRecord()

                if read_strand == '1':
                    round_2_MappingRecord_dict[read_id_base].r1_refs[ref_id_with_pos] = cigar
                    round_2_MappingRecord_dict[read_id_base].r1_cigar_to_flag[cigar] = read_flag
                    round_2_MappingRecord_dict[read_id_base].r1_filtered_refs.add(ref_id)

                if read_strand == '2':
                    round_2_MappingRecord_dict[read_id_base].r2_refs[ref_id_with_pos] = cigar
                    round_2_MappingRecord_dict[read_id_base].r2_cigar_to_flag[cigar] = read_flag
                    round_2_MappingRecord_dict[read_id_base].r2_filtered_refs.add(ref_id)

                if clipping_len >= min_clp_len:
                    store_read_seq = True
            else:
                store_read_seq = True
        else:
            store_read_seq = True

        # store_read_seq into dict
        if store_read_seq is True:

            # turn back if read reverse complemented
            read_rc = sam_flag_to_rc(read_flag)
            read_seq_to_store = read_seq
            if read_rc is True:
                read_seq_to_store = get_rc(read_seq)

            if read_id_base not in round_2_MappingRecord_dict:
                round_2_MappingRecord_dict[read_id_base] = MappingRecord()
            if read_strand == '1':
                if round_2_MappingRecord_dict[read_id_base].r1_seq == '':
                    round_2_MappingRecord_dict[read_id_base].r1_seq = read_seq_to_store
            if read_strand == '2':
                if round_2_MappingRecord_dict[read_id_base].r2_seq == '':
                    round_2_MappingRecord_dict[read_id_base].r2_seq = read_seq_to_store

# parse round_2_MappingRecord_dict
free_living_ctg_refs_file_handle = open(free_living_ctg_ref_file, 'w')
free_living_ctg_R1_handle = open(free_living_ctg_R1, 'w')
free_living_ctg_R2_handle = open(free_living_ctg_R2, 'w')
free_living_ctg_UP_handle = open(free_living_ctg_UP, 'w')
for read_basename in round_2_MappingRecord_dict.copy():
    read_mr = round_2_MappingRecord_dict[read_basename]
    r1_ref_cigar_list = list(read_mr.r1_refs.values())
    r2_ref_cigar_list = list(read_mr.r2_refs.values())
    best_cigar, max_clp, max_clp_location = get_max_clp_and_index(r1_ref_cigar_list, r2_ref_cigar_list)

    if (read_mr.r1_refs == {}) and (read_mr.r2_refs != {}):

        if len(read_mr.r2_refs) == 1:

            r2_ref_cigar = r2_ref_cigar_list[0]
            r2_ref_flag = read_mr.r2_cigar_to_flag[r2_ref_cigar]
            r2_ref_cigar_rc = sam_flag_to_rc(r2_ref_flag)

            # consider the unmapped mate only
            if max_clp < min_clp_len:
                read_mr.consider_round_2 = True
                read_mr.consider_r2_unmapped_mate = True

                # write out refs
                free_living_ctg_refs_file_handle.write('%s.1\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))

                # write out sequence
                free_living_ctg_UP_handle.write('>%s.1\n' % read_basename)
                free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_seq)

            # consider both of unmapped mate and clipping part
            else:
                r2_ref_no_pos = list(read_mr.r2_refs.keys())[0].split('_pos_')[0]
                r2_ref_pos = int(list(read_mr.r2_refs.keys())[0].split('_pos_')[1])

                if (r2_ref_no_pos[-1]) == (max_clp_location[-1]) == 'l':
                    read_mr.consider_round_2 = True
                    read_mr.consider_r2_unmapped_mate = True
                    if clp_read_for_assembly is True:
                        read_mr.consider_r2_clipping_part = True

                    # write out refs
                    free_living_ctg_refs_file_handle.write(
                        '%s.1\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))
                    if clp_read_for_assembly is True:
                        free_living_ctg_refs_file_handle.write(
                            '%s.2\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))

                    # if r2_ref_cigar_rc is False:
                    #     read_mr.r2_clipping_seq = read_mr.r2_seq[:max_clp]
                    # if r2_ref_cigar_rc is True:
                    #     r2_seq_rc = get_rc(read_mr.r2_seq)
                    #     r2_clipping_seq_rc = r2_seq_rc[:max_clp]
                    #     read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)

                elif (r2_ref_no_pos[-1]) == (max_clp_location[-1]) == 'r':
                    read_mr.consider_round_2 = True
                    read_mr.consider_r2_unmapped_mate = True
                    if clp_read_for_assembly is True:
                        read_mr.consider_r2_clipping_part = True

                    # write out refs
                    free_living_ctg_refs_file_handle.write(
                        '%s.1\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))
                    if clp_read_for_assembly is True:
                        free_living_ctg_refs_file_handle.write(
                            '%s.2\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))

                    # if r2_ref_cigar_rc is False:
                    #     read_mr.r2_clipping_seq = read_mr.r2_seq[-max_clp:]
                    # if r2_ref_cigar_rc is True:
                    #     r2_seq_rc = get_rc(read_mr.r2_seq)
                    #     r2_clipping_seq_rc = r2_seq_rc[-max_clp:]
                    #     read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)

                else:  # mapped to unwanted end, ignore
                    round_2_MappingRecord_dict.pop(read_basename)

                # write out sequence
                if read_mr.consider_round_2 is True:
                    # write out R1 fa
                    free_living_ctg_R1_handle.write('>%s.1\n' % read_basename)
                    free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_seq)
                    # write out R2 fa
                    if clp_read_for_assembly is True:
                        free_living_ctg_R2_handle.write('>%s.2\n' % read_basename)
                        # free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_clipping_seq)
                        free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_seq)

        else:  # r2 mapped to multiple refs, ignore
            round_2_MappingRecord_dict.pop(read_basename)

    elif (read_mr.r1_refs != {}) and (read_mr.r2_refs == {}):

        if len(read_mr.r1_refs) == 1:

            r1_ref_cigar = r1_ref_cigar_list[0]
            r1_ref_flag = read_mr.r1_cigar_to_flag[r1_ref_cigar]
            r1_ref_cigar_rc = sam_flag_to_rc(r1_ref_flag)

            # consider the unmapped mate only
            if max_clp < min_clp_len:
                read_mr.consider_round_2 = True
                read_mr.consider_r1_unmapped_mate = True

                # write out refs
                free_living_ctg_refs_file_handle.write(
                    '%s.2\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))

                # write out sequence
                free_living_ctg_UP_handle.write('>%s.2\n' % read_basename)
                free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_seq)

            # consider both of unmapped mate and clipping part
            else:
                r1_ref_no_pos = list(read_mr.r1_refs.keys())[0].split('_pos_')[0]
                r1_ref_pos = int(list(read_mr.r1_refs.keys())[0].split('_pos_')[1])

                if (r1_ref_no_pos[-1]) == (max_clp_location[-1]) == 'l':
                    read_mr.consider_round_2 = True
                    read_mr.consider_r1_unmapped_mate = True
                    if clp_read_for_assembly is True:
                        read_mr.consider_r1_clipping_part = True

                    # write out refs
                    if clp_read_for_assembly is True:
                        free_living_ctg_refs_file_handle.write(
                            '%s.1\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))
                    free_living_ctg_refs_file_handle.write(
                        '%s.2\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))

                    # if r1_ref_cigar_rc is False:
                    #     read_mr.r1_clipping_seq = read_mr.r1_seq[:max_clp]
                    # if r1_ref_cigar_rc is True:
                    #     r1_seq_rc = get_rc(read_mr.r1_seq)
                    #     r1_clipping_seq_rc = r1_seq_rc[:max_clp]
                    #     read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)

                elif (r1_ref_no_pos[-1]) == (max_clp_location[-1]) == 'r':
                    read_mr.consider_round_2 = True
                    read_mr.consider_r1_unmapped_mate = True
                    if clp_read_for_assembly is True:
                        read_mr.consider_r1_clipping_part = True

                    # write out refs
                    if clp_read_for_assembly is True:
                        free_living_ctg_refs_file_handle.write(
                            '%s.1\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))
                    free_living_ctg_refs_file_handle.write(
                        '%s.2\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))

                    # if r1_ref_cigar_rc is False:
                    #     read_mr.r1_clipping_seq = read_mr.r1_seq[-max_clp:]
                    # if r1_ref_cigar_rc is True:
                    #     r1_seq_rc = get_rc(read_mr.r1_seq)
                    #     r1_clipping_seq_rc = r1_seq_rc[-max_clp:]
                    #     read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)

                else:  # mapped to unwanted end, ignore
                    round_2_MappingRecord_dict.pop(read_basename)

                # write out sequence
                if read_mr.consider_round_2 is True:
                    # write out R1 fa
                    if clp_read_for_assembly is True:
                        free_living_ctg_R1_handle.write('>%s.1\n' % read_basename)
                        # free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_clipping_seq)
                        free_living_ctg_R1_handle.write('%s\n' % read_mr.r1_seq)
                    # write out R2 fa
                    free_living_ctg_R2_handle.write('>%s.2\n' % read_basename)
                    free_living_ctg_R2_handle.write('%s\n' % read_mr.r2_seq)

        else:  # r1 mapped to multiple refs, ignore
            round_2_MappingRecord_dict.pop(read_basename)

    elif (read_mr.r1_refs != {}) and (read_mr.r2_refs != {}):
        if max_clp >= min_M_len:
            if clp_read_for_assembly is True:

                if (len(read_mr.r1_refs) == 1) and (len(read_mr.r2_refs) == 1):
                    r1_ref_no_pos = list(read_mr.r1_refs.keys())[0].split('_pos_')[0]
                    r2_ref_no_pos = list(read_mr.r2_refs.keys())[0].split('_pos_')[0]
                    r1_ref_pos = int(list(read_mr.r1_refs.keys())[0].split('_pos_')[1])
                    r2_ref_pos = int(list(read_mr.r2_refs.keys())[0].split('_pos_')[1])

                    if r1_ref_no_pos == r2_ref_no_pos:
                        if (r1_ref_no_pos[-1]) == (r2_ref_no_pos[-1]) == (max_clp_location[-1]):

                            if (max_clp_location == 'r1_l') and (r1_ref_pos == 1):
                                read_mr.consider_round_2 = True
                                read_mr.consider_r1_clipping_part = True

                                # write out refs
                                free_living_ctg_refs_file_handle.write('%s.1\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))

                                # best_cigar_flag = read_mr.r1_cigar_to_flag[best_cigar]
                                # best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                                # if best_cigar_rc is False:
                                #     read_mr.r1_clipping_seq = read_mr.r1_seq[:max_clp]
                                # if best_cigar_rc is True:
                                #     r1_seq_rc = get_rc(read_mr.r1_seq)
                                #     r1_clipping_seq_rc = r1_seq_rc[:max_clp]
                                #     read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)

                            elif (max_clp_location == 'r2_l') and (r2_ref_pos == 1):
                                read_mr.consider_round_2 = True
                                read_mr.consider_r2_clipping_part = True

                                # write out refs
                                free_living_ctg_refs_file_handle.write('%s.2\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))

                                # best_cigar_flag = read_mr.r2_cigar_to_flag[best_cigar]
                                # best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                                # if best_cigar_rc is False:
                                #     read_mr.r2_clipping_seq = read_mr.r2_seq[:max_clp]
                                # if best_cigar_rc is True:
                                #     r2_seq_rc = get_rc(read_mr.r2_seq)
                                #     r2_clipping_seq_rc = r2_seq_rc[:max_clp]
                                #     read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)

                            elif (max_clp_location == 'r1_r') and (r1_ref_pos >= (end_seq_len / 5)):
                                read_mr.consider_round_2 = True
                                print(read_mr.r1_refs)
                                # read_mr.consider_r1_clipping_part = True

                                # write out refs
                                free_living_ctg_refs_file_handle.write('%s.1\t%s\n' % (read_basename, ','.join(read_mr.r1_filtered_refs)))

                                # best_cigar_flag = read_mr.r1_cigar_to_flag[best_cigar]
                                # best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                                # if best_cigar_rc is False:
                                #     read_mr.r1_clipping_seq = read_mr.r1_seq[-max_clp:]
                                # if best_cigar_rc is True:
                                #     r1_seq_rc = get_rc(read_mr.r1_seq)
                                #     r1_clipping_seq_rc = r1_seq_rc[-max_clp:]
                                #     read_mr.r1_clipping_seq = get_rc(r1_clipping_seq_rc)

                            elif (max_clp_location == 'r2_r') and (r2_ref_pos >= (end_seq_len / 5)):
                                read_mr.consider_round_2 = True
                                read_mr.consider_r2_clipping_part = True

                                # write out refs
                                free_living_ctg_refs_file_handle.write(
                                    '%s.2\t%s\n' % (read_basename, ','.join(read_mr.r2_filtered_refs)))

                                # best_cigar_flag = read_mr.r2_cigar_to_flag[best_cigar]
                                # best_cigar_rc = sam_flag_to_rc(best_cigar_flag)

                                # if best_cigar_rc is False:
                                #     read_mr.r2_clipping_seq = read_mr.r2_seq[-max_clp:]
                                # if best_cigar_rc is True:
                                #     r2_seq_rc = get_rc(read_mr.r2_seq)
                                #     r2_clipping_seq_rc = r2_seq_rc[-max_clp:]
                                #     read_mr.r2_clipping_seq = get_rc(r2_clipping_seq_rc)

                            else:  # not too many of them, ignore now, maybe worth check later,
                                round_2_MappingRecord_dict.pop(read_basename)
                                # print('%s\tr1_refs:\t%s\t%s' % (read_basename, read_mr.r1_refs, read_mr.r1_seq))
                                # print('%s\tr2_refs:\t%s\t%s' % (read_basename, read_mr.r2_refs, read_mr.r2_seq))

                            # write out sequence
                            if read_mr.consider_round_2 is True:

                                if read_mr.consider_r1_clipping_part is True:
                                    free_living_ctg_UP_handle.write('>%s.1\n' % read_basename)
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r1_seq)

                                if read_mr.consider_r2_clipping_part is True:
                                    free_living_ctg_UP_handle.write('>%s.2\n' % read_basename)
                                    free_living_ctg_UP_handle.write('%s\n' % read_mr.r2_seq)

                        else:  # mapped to unwanted end, ignore
                            round_2_MappingRecord_dict.pop(read_basename)

                    else:  # r1 and r2 mapped to different refs
                        # not too many of them, ignore now
                        round_2_MappingRecord_dict.pop(read_basename)

                else:  # r1 or r2 mapped to multiple refs
                    # not too many of them, ignore now
                    round_2_MappingRecord_dict.pop(read_basename)

        else:  # ignore and remove element from dict
            round_2_MappingRecord_dict.pop(read_basename)

    else:  # ignore and remove element from dict
        round_2_MappingRecord_dict.pop(read_basename)

free_living_ctg_R1_handle.close()
free_living_ctg_R2_handle.close()
free_living_ctg_UP_handle.close()
free_living_ctg_refs_file_handle.close()





'''MarkerGene	GenomicSeq	Linkage	Step
3_GI_subsample_50_875	Refined_7	2603	S1
3_GI_subsample_100_1832	Refined_7	2508	S1
3_GI_subsample_50_881	Refined_7	2062	S1
3_GI_subsample_100_2289	Refined_7	1791	S1
3_GI_subsample_75_69	Refined_4	1783	S1
3_GI_subsample_100_1336	Refined_4	1752	S1
3_GI_subsample_50_650	Refined_4	1310	S1
3_GI_subsample_75_1732	Refined_7	1303	S1
3_GI_subsample_100_3665	Refined_7	1187	S1
3_GI_subsample_50_882	Refined_7	1173	S1
3_GI_subsample_25_913	Refined_4	924	S1
3_GI_subsample_50_1564	Refined_4	695	S1
3_GI_subsample_75_964	Refined_52	477	S1
3_GI_subsample_50_583	Refined_52	465	S1
3_GI_subsample_75_1207	Refined_43	424	S1
3_GI_subsample_75_304	Refined_29	417	S1
3_GI_subsample_50_580	Refined_52	399	S1
3_GI_subsample_100_3361	Refined_4	387	S1
3_GI_subsample_75_1085	Refined_52	378	S1
3_GI_subsample_50_213	Refined_29	366	S1
3_GI_subsample_100_1366	Refined_42	357	S1
3_GI_subsample_100_1167	Refined_42	340	S1
3_GI_subsample_50_1690	Refined_33	338	S1
3_GI_subsample_75_383	Refined_33	335	S1
3_GI_subsample_75_382	Refined_33	333	S1
3_GI_subsample_75_1091	Refined_43	332	S1
3_GI_subsample_100_1372	Refined_42	331	S1
3_GI_subsample_75_960	Refined_33	297	S1
3_GI_subsample_100_1271	Refined_42	287	S1
3_GI_subsample_75_963	Refined_52	286	S1
3_GI_subsample_75_1206	Refined_43	251	S1
3_GI_subsample_75_1105	Refined_43	249	S1
3_GI_subsample_100_1367	Refined_42	241	S1
3_GI_subsample_25_345	Refined_4	195	S1
3_GI_subsample_100_1437	Refined_31	188	S1
3_GI_subsample_100_1337	Refined_42	183	S1
3_GI_subsample_50_595	Refined_4	173	S1
3_GI_subsample_100_2493	Refined_33	171	S1
3_GI_subsample_75_1086	Refined_52	170	S1
3_GI_subsample_100_2322	Refined_4	168	S1
3_GI_subsample_75_1227	Refined_4	155	S1
3_GI_subsample_75_1075	Refined_31	155	S1
3_GI_subsample_100_1076	Refined_30	147	S1
3_GI_subsample_100_1848	Refined_30	143	S1
3_GI_subsample_100_1847	Refined_30	142	S1
3_GI_subsample_75_1074	Refined_31	129	S1
3_GI_subsample_10_76	Refined_33	127	S1
3_GI_subsample_100_121	Refined_62	121	S1
3_GI_subsample_100_1365	Refined_42	116	S1
3_GI_subsample_50_81	Refined_18	116	S1
3_GI_subsample_75_1078	Refined_31	113	S1
3_GI_subsample_75_1079	Refined_31	107	S1
3_GI_subsample_100_3378	Refined_29	103	S1
3_GI_subsample_25_275	Refined_70	103	S1
3_GI_subsample_100_1192	Refined_11	98	S1
3_GI_subsample_100_1193	Refined_11	98	S1
3_GI_subsample_75_1076	Refined_31	96	S1
3_GI_subsample_75_1195	Refined_52	92	S1
3_GI_subsample_75_961	Refined_52	91	S1
3_GI_subsample_100_317	Refined_2	91	S1
3_GI_subsample_75_188	Refined_2	82	S1
3_GI_subsample_50_424	Refined_42	80	S1
3_GI_subsample_25_484	Refined_95	78	S1
3_GI_subsample_50_663	Refined_31	78	S1
3_GI_subsample_50_113	Refined_21	68	S1
3_GI_subsample_100_1449	Refined_52	67	S1
3_GI_subsample_25_329	Refined_11	67	S1
3_GI_subsample_75_1381	Refined_95	67	S1
3_GI_subsample_25_402	Refined_31	66	S1
3_GI_subsample_50_1037	Refined_52	64	S1
3_GI_subsample_100_1191	Refined_11	63	S1
3_GI_subsample_100_4224	Refined_4	63	S1
3_GI_subsample_25_72	Refined_21	62	S1
3_GI_subsample_100_1338	Refined_42	58	S1
3_GI_subsample_25_582	Refined_52	57	S1
3_GI_subsample_100_313	Refined_2	54	S1
3_GI_subsample_75_226	Refined_2	53	S1
3_GI_subsample_100_279	Refined_2	49	S1
3_GI_subsample_100_3525	Refined_42	46	S1
3_GI_subsample_100_3824	Refined_4	46	S1
3_GI_subsample_100_1395	Refined_36	46	S1
3_GI_subsample_50_285	Refined_25	44	S1
3_GI_subsample_100_1273	Refined_33	43	S1
3_GI_subsample_75_34	Refined_12	43	S1
3_GI_subsample_50_346	Refined_12	43	S1
3_GI_subsample_75_139	Refined_13	40	S1
3_GI_subsample_100_690	Refined_58	40	S1
3_GI_subsample_100_1610	Refined_33	39	S1
3_GI_subsample_75_223	Refined_2	38	S1
3_GI_subsample_100_3152	Refined_42	37	S1
3_GI_subsample_50_811	Refined_95	37	S1
3_GI_subsample_75_1008	Refined_4	36	S1
3_GI_subsample_75_1025	Refined_42	36	S1
3_GI_subsample_100_1393	Refined_36	35	S1
3_GI_subsample_100_187	Refined_2	34	S1
3_GI_subsample_50_869	Refined_95	34	S1
3_GI_subsample_75_1621	Refined_52	33	S1
3_GI_subsample_75_1471	Refined_42	32	S1
3_GI_subsample_75_106	Refined_44	30	S1
3_GI_subsample_75_363	Refined_80	29	S1
3_GI_subsample_75_241	Refined_74	28	S1
3_GI_subsample_100_589	Refined_3	28	S1
3_GI_subsample_25_431	Refined_31	27	S1
3_GI_subsample_100_2066	Refined_4	27	S1
3_GI_subsample_75_2805	Refined_4	26	S1
3_GI_subsample_50_630	Refined_42	25	S1
3_GI_subsample_50_629	Refined_42	25	S1
3_GI_subsample_25_598	Refined_31	25	S1
3_GI_subsample_75_956	Refined_42	24	S1
3_GI_subsample_100_235	Refined_35	23	S1
3_GI_subsample_100_3003	Refined_78	23	S1
3_GI_subsample_10_175	Refined_42	23	S1
3_GI_subsample_75_1701	Refined_76	23	S1
3_GI_subsample_75_1572	Refined_4	22	S1
3_GI_subsample_100_2329	Refined_31	22	S1
3_GI_subsample_50_712	Refined_31	22	S1
3_GI_subsample_100_3364	Refined_4	22	S1
3_GI_subsample_25_863	Refined_4	21	S1
3_GI_subsample_25_763	Refined_4	21	S1
3_GI_subsample_5_85	Refined_59	20	S1
3_GI_subsample_25_565	Refined_4	19	S1
3_GI_subsample_75_1057	Refined_31	18	S1
3_GI_subsample_5_98	Refined_42	18	S1
3_GI_subsample_75_3163	Refined_31	18	S1
3_GI_subsample_100_3344	Refined_42	16	S1
3_GI_subsample_100_2240	Refined_4	16	S1
3_GI_subsample_75_3032	Refined_31	16	S1
3_GI_subsample_50_581	Refined_52	16	S1
3_GI_subsample_75_2486	Refined_42	16	S1
3_GI_subsample_75_2235	Refined_80	15	S1
3_GI_subsample_100_3979	Refined_42	15	S1
3_GI_subsample_50_1945	Refined_4	14	S1
3_GI_subsample_75_2420	Refined_42	14	S1
3_GI_subsample_50_969	Refined_42	14	S1
3_GI_subsample_75_1162	Refined_31	14	S1
3_GI_subsample_75_1055	Refined_31	14	S1
3_GI_subsample_100_4404	Refined_31	13	S1
3_GI_subsample_100_314	Refined_2	13	S1
3_GI_subsample_100_3762	Refined_42	11	S1
3_GI_subsample_100_1548	Refined_72	11	S1
3_GI_subsample_100_2097	Refined_42	11	S1
3_GI_subsample_100_3426	Refined_31	11	S1
3_GI_subsample_100_3714	Refined_31	11	S1
3_GI_subsample_100_1782	Refined_7	11	S1
3_GI_subsample_75_2502	Refined_4	11	S1
3_GI_subsample_50_1728	Refined_31	10	S1
3_GI_subsample_50_370	Refined_69	10	S1
3_GI_subsample_75_1549	Refined_26	10	S1
3_GI_subsample_75_2522	Refined_4	10	S1
3_GI_subsample_100_2560	Refined_2	10	S1
3_GI_subsample_75_2408	Refined_4	9	S1
3_GI_subsample_100_3244	Refined_4	9	S1
3_GI_subsample_100_2419	Refined_2	9	S1
3_GI_subsample_75_1583	Refined_42	9	S1
3_GI_subsample_50_609	Refined_42	8	S1
3_GI_subsample_100_1874	Refined_78	8	S1
3_GI_subsample_100_2390	Refined_42	8	S1
3_GI_subsample_25_378	Refined_42	8	S1
3_GI_subsample_100_4325	Refined_4	8	S1
3_GI_subsample_50_1551	Refined_42	7	S1
3_GI_subsample_100_1396	Refined_36	7	S1
3_GI_subsample_100_1697	Refined_2	7	S1
3_GI_subsample_75_1026	Refined_42	7	S1
3_GI_subsample_50_1500	Refined_4	6	S1
3_GI_subsample_100_2291	Refined_45	6	S1
3_GI_subsample_100_2658	Refined_26	6	S1
3_GI_subsample_50_1600	Refined_31	6	S1
3_GI_subsample_100_4092	Refined_4	6	S1
3_GI_subsample_75_2542	Refined_31	6	S1
3_GI_subsample_50_1595	Refined_42	6	S1
3_GI_subsample_50_2032	Refined_31	5	S1
3_GI_subsample_100_797	Refined_89	5	S1
3_GI_subsample_100_4195	Refined_42	5	S1
3_GI_subsample_75_917	Refined_52	5	S1
3_GI_subsample_100_4380	Refined_42	5	S1
3_GI_subsample_75_2742	Refined_4	5	S1
3_GI_subsample_75_151	Refined_35	5	S1
3_GI_subsample_50_670	Refined_59	5	S1
3_GI_subsample_75_216	Refined_2	5	S1
3_GI_subsample_100_197	Refined_2	5	S1

'''